<?php

/**
 * liste des articles d'une catégorie
 *
 * syntaxe : {up jcontent-by-categories=id-catégorie(s)}
 *
 * Une action très pratique pour lister les articles de la catégorie en cours, il suffit de taper {up article-category}
 *
 * Les mots-clés :
 * ##title## ##title-link## ##subtitle## ##link##
 * ##intro## ##intro-text## ##intro-text,100## ##content##
 * ##image## ##image-src## ##image-alt##
 * ##date-crea## ##date-modif## ##date-publish##
 * ##author## ##note## ##cat## ##cat-link## ##new## ##featured##  ##hits## ##tags-list##
 *
 * @author   LOMART
 * @version  UP-1.7
 * @license   <a href="http://www.gnu.org/licenses/gpl-3.0.html" target="_blank">GNU/GPLv3</a>
 * @tags     Joomla
 */

/**
 * v2.8 ajout mot-clé ##cat-link##
 */
defined('_JEXEC') or die();

class jcontent_by_categories extends upAction
{

    function init()
    {
        // charger les ressources communes à toutes les instances de l'action
        return true;
    }

    function run()
    {
        // lien vers la page de demo (vide=page sur le site de UP)
        $this->set_demopage();

        $options_def = array(
            __class__ => '', // ID(s) catégorie(s) séparé(s) avec virgule, vide pour celle de l'article actuel ou 0 pour toutes
            'maxi' => '', // Nombre maxi d'articles dans la liste. Vide = tous
            'exclude' => '', // liste des id des catégories non reprises si option principale=0
            'current' => '0', // 1 pour inclure l'article en cours
            'no-published' => '1', // Liste aussi les articles non publiés
            'sort-by' => 'publish_up', // tri: title, ordering, created, modified, publish_up, id, hits
            'sort-order' => 'desc', // ordre de tri : asc, desc
            'author' => '', // filtre sur auteur: liste des id ou article, current
            'template' => '', // modèle de mise en page. Si vide le modèle est le contenu
            'main-tag' => 'div', // balise pour le bloc englobant tous les articles. 0 pour aucun
            'main-style' => '', // classes et styles inline pour bloc principal
            'main-class' => '', // classe(s) pour bloc principal (obsolète)
            'item-tag' => 'div', // balise pour le bloc d'un article. 0 pour aucun
            'item-style' => '', // classes et styles inline pour un article
            'item-class' => '', // classe(s) pour un article (obsolète)
            'date-format' => '%e %B %Y', // format pour les dates
            'date-locale' => '', // localisation pour les dates. Par défaut, celle du navigateur client.
            'new-days' => '30', // nombre de jours depuis la création de l'article
            'new-html' => '[span class="badge bg-red t-white"]nouveau[/span]', // code HTML pour badge NEW
            'featured-html' => '&#x2B50 ', // code HTML pour article en vedette
            'no-content-html' => 'Pas de nouvelles, bonnes nouvelles ...[br]No news, good news ...', // texte si aucune correspondance. 0=aucun texte
            'image-src' => '//lorempixel.com/300/300', // image par défaut
            'image-alt' => 'news', // image, texte alternatif par défaut
            'tags-list-prefix' => '', // texte avant les autres éventuels tags
            'tags-list-style' => 'badge;margin-right:4px', // classe ou style affecté à une balise span par mot-clé
            'tags-list-separator' => ' ', // séparateur entre mots-clés
            'id' => '',
            'css-head' => '' // code CSS dans le head
        );

        // si catégorie article non indiquée, c'est celle de l'article
        if ($this->options_user[__class__] === true) {
            if (isset($this->article->catid)) {
                // le shortcode est dans un article
                $this->options_user[__class__] = intval($this->article->catid);
            } else {
                // le shortcode est dans un module
                // TODO : récupérer le catid de l'article actuellement affiché
                return ''; // sortir tant que pas de CATID
            }
        }
        // ======> fusion et controle des options
        $options = $this->ctrl_options($options_def);
        $options['template'] = $this->get_bbcode($options['template'], false);
        $options['new-html'] = $this->get_bbcode($options['new-html'], false);
        $options['no-content-html'] = $this->get_bbcode($options['no-content-html'], false);
        // ======> verif template (modèle de mise en page)
        // en priorité : le sontenu entre shortcode
        // en second : le model dans prefs.ini
        if ($this->content) {
            $tmpl = $this->content;
        } else {
            $tmpl = $options['template'];
        }
        if (! $tmpl) {
            $this->msg_error(JText::_('Aucun contenu ni template'));
            return false;
        }

        // ======> contrôle clé de tri
        $list_sortkey = 'title, ordering, created, modified, publish_up, id, hits';
        $options['sort-by'] = $this->ctrl_argument($options['sort-by'], $list_sortkey);

        // =====> liste des catégories
        $catid = $options[__class__];
        $artid = '';
        if ($catid == '') {
            // la catégorie de l'article en cours
            if (isset($this->article->catid)) {
                // la catégorie de l'article en cours
                $catid = $this->article->catid;
                $artid = $this->article->id;
            } else {
                // appel d'un module : catégorie courante pour la page
                $app = JFactory::getApplication();
                $artid = $app->input->get('id', 0); // v2.5
                $view = $app->input->get('view', 0);
                switch ($view) {
                    case 'article':
                        $database = JFactory::getDBO();
                        $query = "SELECT catid FROM #__content WHERE id=" . $artid;
                        $database->setQuery($query);
                        $row = $database->loadObject();
                        $catid = ($row != null) ? $row->catid : '';
                        break;
                    case 'categories':
                    case 'category':
                        $catid = $artid;
                        break;
                    default:
                        $catid = '';
                        break;
                }
            }
        } elseif ($catid == '0') {
            // toutes les catégories = liste cat de niveau 1 - v2.6
            $catid = '';
            if ($this->J4) {
                $this->categories = JCategories::getInstance('com_content');
                $catniv1 = $this->categories->get()->getChildren();
                foreach ($catniv1 as $cat) {
                    $catid .= $cat->id . ',';
                }
            }
        }
        $catid = explode(',', $catid);

        // =====> RECUP DES DONNEES
        // Get an instance of the generic articles model
        $model = JModelLegacy::getInstance('Articles', 'ContentModel', array(
            'ignore_request' => true
        ));
        if (is_bool($model)) {
            return 'Aucune catégorie';
        }

        // Set application parameters in model
        $app = JFactory::getApplication();
        $appParams = $app->getParams();
        $model->setState('params', $appParams);

        // Set the filters based on the module params
        // nombre d'article
        $model->setState('list.start', 0);
        if ($options['maxi'] != '') {
            $model->setState('list.limit', (int) $options['maxi']);
        }
        // etat publication
        if ($options['no-published'] !== true) {
            $model->setState('filter.published', 1);
        }

        // auteur(s)
        switch ($options['author']) {
            case 'article': // l'auteur de l'article en cours
                $model->setState('filter.author_id', $this->article->created_by);
                break;
            case 'current': // l'utilisateur connecté
                $userid = JFactory::getUser()->id;
                $model->setState('filter.author_id', $userid);
                break;
            default:
                $model->setState('filter.author_id', explode(',', $options['author']));
                break;
        }

        // Access filter
        $access = ! JComponentHelper::getParams('com_content')->get('show_noauth');
        $authorised = JAccess::getAuthorisedViewLevels(JFactory::getUser()->get('id'));
        $model->setState('filter.access', $access);

        // Category filter
        $model->setState('filter.category_id', $catid);
        if ($options['exclude']) {
            // Exclude
            $model->setState('filter.category_id.include', false);
        }

        // article en cours
        if ($artid != '' && $options['current'] != '1') {
            $model->setState('filter.article_id', $artid);
            $model->setState('filter.article_id.include', false); // Exclude
        }
        // Filter by language
        $model->setState('filter.language', $app->getLanguageFilter());

        // Ordering
        $model->setState('list.ordering', 'a.' . $options['sort-by']);
        $model->setState('list.direction', $options['sort-order']);

        $items = $model->getItems();

        if (count($items) == 0)
            return ($options['no-content-html'] == '0') ? '' : $options['no-content-html'];

        // ======> Style général et par article
        $main_attr['id'] = $options['id'];
        $this->get_attr_style($main_attr, $options['main-class'], $options['main-style']);
        $this->get_attr_style($sItem_attr, $options['item-class'], $options['item-style']);
        $this->get_attr_style($tags_list_attr, $options['tags-list-style']);
        // format date
        $this->set_locale($options['date-locale']);

        // css-head
        $this->load_css_head($options['css-head']);

        // ======> mise en forme résultat
        if ($options['main-tag'] != '0') {
            $html[] = $this->set_attr_tag($options['main-tag'], $main_attr);
        }
        foreach ($items as $item) {
            // --- Bloc article
            if ($options['item-tag'] != '0')
                $html[] = $this->set_attr_tag($options['item-tag'], $sItem_attr);
            $sItem = $tmpl; // reinit pour nouvel article
                            // --- lien vers l'article
            $url = '';
            $slug = ($item->alias) ? ($item->id . ':' . $item->alias) : $itemid;
            $catslug = ($item->category_alias) ? ($item->catid . ':' . $item->category_alias) : $item->catid;
            $route = ContentHelperRoute::getArticleRoute($slug, $catslug);
            $url = JRoute::_($route);
            // --- le titre et sous titre
            $title = $item->title;
            $subtitle = '';
            if (stripos($sItem, '##subtitle##') !== false) {
                $title = strstr($item->title . '~', '~', true);
                $subtitle = trim(substr(strstr($item->title, '~'), 1));
            }
            // --- traitement $item->introtext et $item->fulltext
            // si pas d'introtext -> introtext=fulltext et fulltext=vide
            // ce n'est pas souhaitable car le contenu fulltext n'est pas prévu pour cela
            if ($item->fulltext == '') {
                $item->fulltext = $item->introtext;
                $item->introtext = '';
            }
            // ==== les remplacements
            // {id} : ID de l'article
            $sItem = str_ireplace('##id##', $item->id, $sItem);
            // {link} : lien vers l'article - a mettre dans balise a
            $sItem = str_ireplace('##link##', $url, $sItem);
            // {title-link} : titre de l'article
            $sItem = str_ireplace('##title-link##', '<a href="' . $url . '">' . $title . '</a>', $sItem);
            // {title} : titre de l'article
            $sItem = str_ireplace('##title##', $title, $sItem);
            // {subtitle} : sous-titre de l'article (partie après tilde du titre)
            $sItem = str_ireplace('##subtitle##', $subtitle, $sItem);
            // {alias} : alias de l'article v1.8
            $sItem = str_ireplace('##alias##', $item->alias, $sItem);
            // {cat} : catégorie de l'article
            // {date-crea} : date de création
            $sItem = str_ireplace('##date-crea##', utf8_encode(strftime($options['date-format'], strtotime($item->created))), $sItem);
            // {date-modif} : date de création
            $sItem = str_ireplace('##date-crea##', utf8_encode(strftime($options['date-format'], strtotime($item->modified))), $sItem);
            // {date-publish} : date de création
            $sItem = str_ireplace('##date-publish##', utf8_encode(strftime($options['date-format'], strtotime($item->publish_up))), $sItem);
            // {author} : auteur
            $sItem = str_ireplace('##author##', $item->author, $sItem);
            // {intro} : texte d'introduction en HTML
            $sItem = str_ireplace('##intro##', $item->introtext, $sItem);
            $sItem = str_ireplace('##content##', $item->fulltext, $sItem); // v2.4
                                                                           // {note} : note sur l'article
                                                                           // TODO : trouver une méthode moins bourrin !!!
            if (stripos($sItem, '##note##') !== false) {
                $db = JFactory::getDbo();
                $query = $db->getQuery(true);
                $query->select('note')
                    ->from('#__content')
                    ->where('id = ' . $item->id);
                $db->setQuery($query);
                $result = $db->loadResult();
                $sItem = str_ireplace('##note##', $result, $sItem);
            }
            // {cat} : nom catégorie
            $sItem = str_ireplace('##cat##', $item->category_title, $sItem);
            // 
            if (strpos($sItem,'##cat-link##')) {
                $caturl = JRoute::_(ContentHelperRoute::getCategoryRoute($catslug));
                $sItem = str_ireplace('##cat-link##', '<a href="' . $caturl . '">' . $item->category_title . '</a>', $sItem);
            }
            // {tags-list} : liste des tags
            if (stripos($sItem, '##tags-list##') !== false) {
                $db = JFactory::getDbo();
                $query = $db->getQuery(true);
                $query->select('t.title')
                    ->from('#__tags as t')
                    ->innerJoin('#__contentitem_tag_map as m on t.id = m.tag_id')
                    ->where('m.content_item_id = ' . $item->id . ' AND m.type_alias like "%article%"');
                $db->setQuery($query);
                $listTags = $db->loadObjectList();
                $tmpTags = (empty($listTags)) ? '' : $options['tags-list-prefix'];
                $tmpTag = array();
                foreach ($listTags as $tag) {
                    if ($options['tags-list-style'] != '') {
                        $tmpTag[] = $this->set_attr_tag('span', $tags_list_attr, $tag->title);
                    } else {
                        $tmpTag[] = $tag->title;
                    }
                }
                $sItem = str_ireplace('##tags-list##', implode($options['tags-list-separator'], $tmpTag), $sItem);
            }
            // {featured} : en vedette
            if (stripos($sItem, '##featured##') !== false) {
                $tmp = ($item->featured == '1') ? $options['featured-html'] : '';
                $sItem = str_ireplace('##featured##', $tmp, $sItem);
            }
            // {new} : badge
            if (stripos($sItem, '##new##') !== false) {
                $max = date('Y-m-d H:i:s', mktime(date("H"), date("i"), 0, date("m"), date("d") - intval($options['new-days']), date("Y")));
                $new = ($item->publish_up > $max) ? $options['new-html'] : '';
                $sItem = str_ireplace('##new##', $new, $sItem);
            }
            // {hits}
            $sItem = str_ireplace('##hits##', $item->hits, $sItem);
            // {image-xxx} : l'image d'intro, sinon celle dans l'introtext
            // {image} : la balise img complete
            // {image-src} et {image-alt} : uniquement src et alt d'une balise img existante
            if (stripos($sItem, '##image') !== false) {
                $images = json_decode($item->images);
                $img_src = $options['image-src'];
                $img_alt = $options['image-alt'];
                if ($images->image_intro) {
                    $img_src = $images->image_intro;
                    $img_alt = $images->image_intro_alt;
                } else {
                    $imgTag = $this->preg_string('#(\<img .*\>)#Ui', $item->introtext);
                    if ($imgTag) {
                        $imgAttr = $this->get_attr_tag($imgTag, 'alt');
                        $img_src = $imgAttr['src'];
                        $img_alt = $imgAttr['alt'];
                    }
                }
                $img_tag = '<img src="' . $img_src . '" alt="' . $img_alt . '">';
                $sItem = str_ireplace('##image##', $img_tag, $sItem);
                $sItem = str_ireplace('##image-src##', $img_src, $sItem);
                $sItem = str_ireplace('##image-alt##', $img_alt, $sItem);
            }

            // --- tags avec param
            // {intro-text,100} : les 100 premiers caractères de l'introduction en texte brut
            preg_match('#\#\#intro-text\s*,?\s*([0-9]*)\s*\#\##Ui', $sItem, $tag);
            if (isset($tag[0])) {
                $intro = trim(strip_tags($item->introtext));
                if (isset($tag[1]) && $tag[1]) {
                    $len = (int) $tag[1];
                    if (strlen($intro) > $len)
                        $intro = mb_substr($intro, 0, $len) . '...';
                }
                $sItem = str_replace($tag[0], $intro, $sItem);
            }
            // --- fin article
            $html[] = $sItem;
            if ($options['item-tag'] != '0')
                $html[] = '</' . $options['item-tag'] . '>';
        }
        if ($options['main-tag'] != '0')
            $html[] = '</' . $options['main-tag'] . '>';
        return implode(PHP_EOL, $html);
    }

    // run
}

// class






